﻿-----------------------------------------------------------------------
-- Locale
-----------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Violation: DPS")

L:RegisterTranslations("enUS", function() return {
	["Damage per Second"] = true,
	["DPS"] = true,
} end)

L:RegisterTranslations("zhCN", function() return {
	["Damage per Second"] = "每秒伤害",
	["DPS"] = "每秒伤害(DPS)",	
} end)
	
L:RegisterTranslations("zhTW", function() return {
	["Damage per Second"] = "每秒傷害",
	["DPS"] = "每秒傷害(DPS)",
} end)
	
L:RegisterTranslations("koKR", function() return {
	["Damage per Second"] = "초당 피해량",
} end)

L:RegisterTranslations("frFR", function() return {
	["Damage per Second"] = "Dégâts par seconde",
	["DPS"] = "DPS",
} end)

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local seconds = nil
local damageDone = nil
local moduleName = "Damage per Second"
local isInCombat = {}
local combatStartTime = {}
local lastShotTime = {}
local isInCombat = {}

local COMBAT_TIMEOUT = 4

local roster = AceLibrary("Roster-2.1")

local parserEvent = {
	eventType = "Damage",
	sourceID_not = false,
}

-----------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------

local mod = Violation:NewModule(moduleName)
mod.revision = tonumber(("$Revision: 66014 $"):sub(12, -3)) or 1
mod.version = 5
mod.hidePercentage = true
mod.violationCategory = "damage"

-----------------------------------------------------------------------
-- Module initialization
-----------------------------------------------------------------------

function mod:OnModuleEnable()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "OnEvent")
	self:RegisterEvent("PLAYER_REGEN_ENABLED") -- handle player dps time via regen enabled
	self:ScheduleRepeatingEvent( self.CheckPartyCombat, 1, self )
end

function mod:OnReset()
	if type(seconds) == "table" then
		seconds = mod:del(seconds)
	end
	seconds = mod:new()
	
	if type(damageDone) == "table" then
		damageDone = mod:del(damageDone)
	end
	damageDone = mod:new()

	for k in pairs(combatStartTime) do
		combatStartTime[k] = nil
	end
	for k in pairs(lastShotTime) do
		lastShotTime[k] = nil
	end
end

-----------------------------------------------------------------------
-- Module implementation
-----------------------------------------------------------------------

function mod:GetDisplayName() return L["Damage per Second"] end
function mod:GetShortDisplayName() return L["DPS"] end

function mod:OnRemoteData(name, input)
	if name and input then
		self.dirty = (input ~= (self.data[name] or 0))
		self.data[name] = input
	end
end

function mod:OnData(name, input)
	if not self:IsLocal(name) and self.data[name] then
		return
	end	
	self.data[name] = nil
	self.dirty = true
end

function mod:IsDirty()
	return self.dirty or next(isInCombat)
end

function mod:IsBroadcastDirty( charName )
	if not charName then
		charName = self.playerName
	end
	return ( self.broadcast[charName] ~= self:GetData(charName) ) or isInCombat[charName]
end

local function getDuration(name)
	local duration = seconds[name] or 0
	if combatStartTime[name] then
		duration = duration + math.max(lastShotTime[name] - combatStartTime[name], 1.5)
	end
	return math.max(duration, 1.5) 
end

local function calculateData(name)
	return math.floor((damageDone[name] or 0) / getDuration(name))
end

local moduleData = {}
function mod:GetData(name)
	if name then
		return self.data[name] or calculateData(name)
	else
		for k, v in pairs( moduleData ) do
			moduleData[k] = nil
		end
		for name in pairs(damageDone) do
			moduleData[name] = calculateData(name)
		end
		for name, value in pairs(self.data) do
			moduleData[name] = value 
		end
		return moduleData
	end
end

function mod:GetBroadcastData( charName )
	if not charName then
		charName = self.playerName
	end
	self.broadcast[charName] = self:GetData(charName)
	return self.broadcast[charName]	
end

function mod:UpdateDPS(name)
	self:OnData(name, true)
end

function mod:ResetDPSTimer(name)
	if combatStartTime[name] then
		seconds[name] = getDuration(name)
		
		combatStartTime[name] = nil
		lastShotTime[name] = nil
		
		self:UpdateDPS(name)
	end	
end

function mod:CheckDPSTimer(name)
	if lastShotTime[name] and lastShotTime[name] + COMBAT_TIMEOUT < GetTime() then
		self:ResetDPSTimer(name)
	end
end

local COMBATLOG_OBJECT_AFFILIATION_MINE		= COMBATLOG_OBJECT_AFFILIATION_MINE		or 0x00000001
local COMBATLOG_OBJECT_AFFILIATION_PARTY	= COMBATLOG_OBJECT_AFFILIATION_PARTY	or 0x00000002
local COMBATLOG_OBJECT_AFFILIATION_RAID		= COMBATLOG_OBJECT_AFFILIATION_RAID		or 0x00000004
local COMBATLOG_OBJECT_TYPE_PLAYER			= COMBATLOG_OBJECT_TYPE_PLAYER			or 0x00000400
local LIB_FILTER_PARTY = bit.bor(COMBATLOG_OBJECT_TYPE_PLAYER, COMBATLOG_OBJECT_AFFILIATION_PARTY)
local LIB_FILTER_RAID  = bit.bor(COMBATLOG_OBJECT_TYPE_PLAYER, COMBATLOG_OBJECT_AFFILIATION_RAID)
local LIB_FILTER_GROUP = bit.bor(LIB_FILTER_PARTY, LIB_FILTER_RAID)

function mod:OnEvent(timestamp, eventType, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
	local amount
	if eventType == "SWING_DAMAGE" then
		amount = ...
	elseif eventType == "RANGE_DAMAGE" or eventType == "SPELL_DAMAGE" or eventType == "SPELL_PERIODIC_DAMAGE" or eventType == "DAMAGE_SHIELD" or eventType == "DAMAGE_SPLIT" then
		amount = select(4, ...)
	end
	if amount and amount > 0 then
		local isSmember = bit.band(srcFlags, LIB_FILTER_GROUP)
		isSmember = (isSmember == COMBATLOG_OBJECT_TYPE_PLAYER or isSmember == COMBATLOG_OBJECT_AFFILIATION_PARTY or isSmember == COMBATLOG_OBJECT_AFFILIATION_RAID)
		local isDmember = bit.band(dstFlags, LIB_FILTER_GROUP)
		isDmember = (isDmember == COMBATLOG_OBJECT_TYPE_PLAYER or isDmember == COMBATLOG_OBJECT_AFFILIATION_PARTY or isDmember == COMBATLOG_OBJECT_AFFILIATION_RAID)
		if isSmember and not isDmember then
		 	self:CheckDPSTimer(srcName)
	
		 	local newTime = GetTime()
			combatStartTime[srcName] = combatStartTime[srcName] or newTime
			lastShotTime[srcName] = newTime
			damageDone[srcName] = ( damageDone[srcName] or 0 ) + amount
	
			self:UpdateDPS(srcName)
		end
	end
end

-----------------------------------------------------------------------
-- Combat status watching
-----------------------------------------------------------------------

function mod:PLAYER_REGEN_ENABLED()
	mod:ResetDPSTimer( playerName )
end

function mod:CheckPartyCombat()
	for name, unit in pairs( roster.roster ) do
		if isInCombat[name] then
			if not UnitAffectingCombat( unit.unitid ) then
				isInCombat[name] = nil
				mod:ResetDPSTimer( name )
			else
				mod:CheckDPSTimer( name )
			end
		elseif UnitAffectingCombat( unit.unitid ) then
			isInCombat[name] = true
		end
	end
end
