﻿-----------------------------------------------------------------------
-- Locale
-----------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Violation: DamageTaken")

L:RegisterTranslations("enUS", function() return {
	["Damage taken"] = true,
	["DT"] = true,
} end)

L:RegisterTranslations("zhCN", function() return {
	["Damage taken"] = "承受伤害",
	["DT"] = "承受伤害(DT)",
} end)

L:RegisterTranslations("zhTW", function() return {
	["Damage taken"] = "承受傷害",
	["DT"] = "承受傷害(DT)",
} end)

L:RegisterTranslations("koKR", function() return {
	["Damage taken"] = "받은 피해량",
} end)

L:RegisterTranslations("frFR", function() return {
	["Damage taken"] = "Dégâts reçus",
	["DT"] = "DR",
} end)

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local moduleName = "Damage taken"
local parserEvent = {
	eventType_in = {
		Damage = true,
		Environmental = true,
	},
	recipientID_not = false,
}

-----------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------

local mod = Violation:NewModule(moduleName)
mod.revision = tonumber(("$Revision: 66014 $"):sub(12, -3)) or 1
mod.version = 5
mod.violationCategory = "damage"

-----------------------------------------------------------------------
-- Module initialization
-----------------------------------------------------------------------

function mod:OnModuleEnable()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "OnEvent")
end

-----------------------------------------------------------------------
-- Module implementation
-----------------------------------------------------------------------

function mod:GetDisplayName() return L["Damage taken"] end
function mod:GetShortDisplayName() return L["DT"] end

local COMBATLOG_OBJECT_AFFILIATION_MINE		= COMBATLOG_OBJECT_AFFILIATION_MINE		or 0x00000001
local COMBATLOG_OBJECT_AFFILIATION_PARTY	= COMBATLOG_OBJECT_AFFILIATION_PARTY	or 0x00000002
local COMBATLOG_OBJECT_AFFILIATION_RAID		= COMBATLOG_OBJECT_AFFILIATION_RAID		or 0x00000004
local COMBATLOG_OBJECT_TYPE_PLAYER			= COMBATLOG_OBJECT_TYPE_PLAYER			or 0x00000400
local LIB_FILTER_PARTY = bit.bor(COMBATLOG_OBJECT_TYPE_PLAYER, COMBATLOG_OBJECT_AFFILIATION_PARTY)
local LIB_FILTER_RAID  = bit.bor(COMBATLOG_OBJECT_TYPE_PLAYER, COMBATLOG_OBJECT_AFFILIATION_RAID)
local LIB_FILTER_GROUP = bit.bor(LIB_FILTER_PARTY, LIB_FILTER_RAID)

function mod:OnEvent(timestamp, eventType, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
	local amount
	if eventType == "SWING_DAMAGE" then
		amount = ...
	elseif eventType == "RANGE_DAMAGE" or eventType == "SPELL_DAMAGE" or eventType == "SPELL_PERIODIC_DAMAGE" or eventType == "DAMAGE_SHIELD" or eventType == "DAMAGE_SPLIT" then
		amount = select(4, ...)
	end
	if amount and amount > 0 then
		local isDmember = bit.band(dstFlags, LIB_FILTER_GROUP)
		isDmember = (isDmember == COMBATLOG_OBJECT_TYPE_PLAYER or isDmember == COMBATLOG_OBJECT_AFFILIATION_PARTY or isDmember == COMBATLOG_OBJECT_AFFILIATION_RAID)
		if isDmember then
			self:OnData(dstName, amount)
		end
	end
end
